<?php

namespace Drupal\membership_commerce\Resolvers;

use Drupal\commerce\Context;
use Drupal\commerce\PurchasableEntityInterface;
use Drupal\commerce_price\Price;
use Drupal\commerce_price\Resolver\PriceResolverInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\membership_term\MembershipTermManagerInterface;

/**
 * BasePriceResolver service.
 */
class AddUserPriceResolver implements PriceResolverInterface {

  /**
   * @var \Drupal\membership_term\MembershipTermManagerInterface
   */
  protected $membershipTermManager;

  /**
   * @param \Drupal\membership_term\MembershipTermManagerInterface $membershipTermManager
   */
  public function __construct(MembershipTermManagerInterface $membershipTermManager) {
    $this->membershipTermManager = $membershipTermManager;
  }

  /**
   * @param \Drupal\commerce\PurchasableEntityInterface $entity
   * @param string $quantity
   * @param \Drupal\commerce\Context $context
   *
   * @return \Drupal\commerce_price\Price|null
   */
  public function resolve(PurchasableEntityInterface $entity, $quantity, Context $context) {
    if ($entity->bundle() == 'membership_add_on') {
      $term = $this->membershipTermManager->getUserActiveMembershipTerm($context->getCustomer());
      $expiration_date = $term->active_dates->end_value;
      //$term->bundle() == 'monthly_membership' | 'annual_membership' | 'quarterly_membership')
      $expires = DrupalDateTime::createFromFormat('Y-m-d', $expiration_date);
      $start = DrupalDateTime::createFromFormat('Y-m-d', $term->active_dates->value);
      $diff = $expires->diff($start);
      $remaining = $expires->diff(new DrupalDateTime());
      $percent_remaining = $remaining->days / $diff->days;

      // Strip out "_membership" from end of bundle name
      $term_length = substr($term->bundle(), 0, -11);
      $variation_storage = \Drupal::entityTypeManager()
        ->getStorage('commerce_product_variation');
      $variations = $variation_storage->loadByProperties(['billing_schedule' => $term_length]);
      /** @var \Drupal\commerce_product\Entity\ProductVariationInterface $variation */
      $variation = reset($variations);
      $per_user_price = $variation->getPrice()->getNumber();
      $prorated_price = $per_user_price * $percent_remaining;
      return new Price($prorated_price, $variation->getPrice()
        ->getCurrencyCode());
    }
    return null;
  }

}
